@@ -91,12 +91,13 @@ Graph representation
 """
 from __future__ import annotations
 
+from functools import cached_property
 from itertools import chain
 from typing import TYPE_CHECKING
 
 import numpy as np
 
-from .constants import INF, S_DEF_DEFAULT, NumberLike
+from .constants import S_DEF_DEFAULT, NumberLike
 from .media import media
 from .network import Network, connect, innerconnect, s2s
 from .util import subplots
@@ -139,7 +140,10 @@ class Circuit:
         except ImportError as err:
             raise ImportError('networkx package as not been installed and is required.') from err
 
-    def __init__(self, connections: list[list[tuple]], name: str = None, auto_reduce: bool = False) -> None:
+    def __init__(self,
+                 connections: list[list[tuple[Network, int]]],
+                 name: str | None = None,
+                 auto_reduce: bool = False) -> None:
         """
         Circuit constructor. Creates a circuit made of a set of N-ports networks.
 
@@ -201,7 +205,7 @@ class Circuit:
 
 
         """
-        self.connections = connections
+        self._connections = connections
         self.name = name
 
         # check if all networks have a name
@@ -230,6 +234,40 @@ class Circuit:
                                               check_duplication=False,
                                               split_ground=True)
 
+    @property
+    def connections(self) -> list[list[tuple[Network, int]]]:
+        """
+        Circuit connections.
+
+        Description of circuit connections. Each connection is a described by a
+        list of tuple. Each tuple contains (network, network_port_nb). Port number
+        indexing starts from zero.
+
+        Returns
+        -------
+        connections : :class:`list[list[tuple[Network, int]]]`
+            The circuit connections.
+
+        """
+        return self._connections
+
+    @connections.setter
+    def connections(self, connections: list[list[tuple[Network, int]]]) -> None:
+        """
+        Set the circuit connections.
+
+        Parameters
+        ----------
+        connections : :class:`list[list[tuple[Network, int]]]`
+            The circuit connections.
+
+        """
+        self._connections = connections
+
+        # Invalidate cached properties
+        for item in ('s', 'X', 'C'):
+            self.__dict__.pop(item, None)
+
     @classmethod
     def check_duplicate_names(cls, connections_list: list):
         """
@@ -264,6 +302,13 @@ class Circuit:
         """
         return ntw._ext_attrs.get("_is_circuit_ground", False)
 
+    @classmethod
+    def _is_open(cls, ntw: Network):
+        """
+        Return True is the network is a open, False otherwise
+        """
+        return ntw._ext_attrs.get("_is_circuit_open", False)
+
     @classmethod
     def Port(cls, frequency: Frequency, name: str, z0: float = 50) -> Network:
         """
@@ -388,11 +433,11 @@ class Circuit:
     @classmethod
     def Ground(cls, frequency: Frequency, name: str, z0: float = 50) -> Network:
         """
-        Return a 2-port network of a grounded link.
+        Return a 1-port network of a grounded link.
 
         Passing the frequency and a name is mandatory.
 
-        The ground link is modelled as an infinite shunt admittance.
+        The ground link is implemented by media.short object.
 
         Parameters
         ----------
@@ -406,7 +451,7 @@ class Circuit:
         Returns
         -------
         ground : :class:`~skrf.network.Network` object
-            2-port network
+            1-port network
 
         Examples
         --------
@@ -420,18 +465,19 @@ class Circuit:
             In [18]: ground = rf.Circuit.Ground(freq, name='GND')
 
         """
-        ground = cls.ShuntAdmittance(frequency, Y=INF, name=name)
+        _media = media.DefinedGammaZ0(frequency, z0=z0)
+        ground = _media.short(name=name)
         ground._ext_attrs['_is_circuit_ground'] = True
         return ground
 
     @classmethod
     def Open(cls, frequency: Frequency, name: str, z0: float = 50) -> Network:
         """
-        Return a 2-port network of an open link.
+        Return a 1-port network of an open link.
 
         Passing the frequency and name is mandatory.
 
-        The open link is modelled as an infinite series impedance.
+        The open link is implemented by media.open object.
 
         Parameters
         ----------
@@ -445,7 +491,7 @@ class Circuit:
         Returns
         -------
         open : :class:`~skrf.network.Network` object
-            2-port network
+            1-port network
 
         Examples
         --------
@@ -459,9 +505,14 @@ class Circuit:
             In [18]: open = rf.Circuit.Open(freq, name='open')
 
         """
-        return cls.SeriesImpedance(frequency, Z=INF, name=name)
+        _media = media.DefinedGammaZ0(frequency, z0=z0)
+        Open = _media.open(name=name)
+        Open._ext_attrs['_is_circuit_open'] = True
+        return Open
 
-    def networks_dict(self, connections: list = None, min_nports: int = 1) -> dict:
+    def networks_dict(self,
+                      connections: list[list[tuple[Network, int]]] | None = None,
+                      min_nports: int = 1) -> dict[str, Network]:
         """
         Return the dictionary of Networks from the connection setup X.
 
@@ -480,13 +531,15 @@ class Circuit:
         if not connections:
             connections = self.connections
 
-        ntws = []
+        ntws: list[Network] = []
         for cnx in connections:
             for (ntw, _port) in cnx:
                 ntws.append(ntw)
         return {ntw.name: ntw for ntw in ntws  if ntw.nports >= min_nports}
 
-    def networks_list(self, connections: list = None, min_nports: int = 1) -> list:
+    def networks_list(self,
+                      connections: list[list[tuple[Network, int]]] | None = None,
+                      min_nports: int = 1) -> list[Network]:
         """
         Return a list of unique networks (sorted by appearing order in connections).
 
@@ -516,7 +569,7 @@ class Circuit:
         return len(self.connections)
 
     @property
-    def connections_list(self) -> list:
+    def connections_list(self) -> list[tuple[int, tuple[Network, int]]]:
         """
         Return the full list of connections, including intersections.
 
@@ -528,7 +581,7 @@ class Circuit:
              ...
             ]
         """
-        return [[idx_cnx, cnx] for (idx_cnx, cnx) in enumerate(chain.from_iterable(self.connections))]
+        return [(idx_cnx, cnx) for (idx_cnx, cnx) in enumerate(chain.from_iterable(self.connections))]
 
     @property
     def networks_nb(self) -> int:
@@ -684,7 +737,7 @@ class Circuit:
         np.einsum('kii->ki', Xs)[:] -= 1  # Sii
         return Xs
 
-    @property
+    @cached_property
     def X(self) -> np.ndarray:
         """
         Return the concatenated intersection matrix [X] of the circuit.
@@ -711,7 +764,7 @@ class Circuit:
 
         return Xf
 
-    @property
+    @cached_property
     def C(self) -> np.ndarray:
         """
         Return the global scattering matrix of the networks.
@@ -748,7 +801,7 @@ class Circuit:
 
         return S  # shape (nb_frequency, nb_inter*nb_n, nb_inter*nb_n)
 
-    @property
+    @cached_property
     def s(self) -> np.ndarray:
         """
         Return the global scattering parameters of the circuit.
@@ -764,7 +817,7 @@ class Circuit:
         return X @ np.linalg.inv(np.identity(self.dim) - self.C @ X)
 
     @property
-    def port_indexes(self) -> list:
+    def port_indexes(self) -> list[int]:
         """
         Return the indexes of the "external" ports.
 
@@ -863,13 +916,14 @@ class Circuit:
         t = np.identity(x.shape[-1]) - c @ x
 
         # Get the sub-matrices of inverse of intermediate temporary matrix t
-        tmp_mat = np.linalg.inv(t[D_idx]) @ t[C_idx]
-        tA_inv = np.linalg.inv(t[A_idx] - t[B_idx] @ tmp_mat)
-        tC_inv = -tmp_mat @ tA_inv
+        # The method np.linalg.solve(A, B) is equivalent to np.inv(A) @ B, but more efficient
+        tmp_mat = np.linalg.solve(t[D_idx], t[C_idx])
 
         # Get the external S-parameters for the external ports
         # Calculated by multiplying the sub-matrices of x and t
-        S_ext = x[A_idx] @ tA_inv + x[B_idx] @ tC_inv
+        S_ext = (x[A_idx] - x[B_idx] @ tmp_mat) @ np.linalg.inv(
+            t[A_idx] - t[B_idx] @ tmp_mat
+        )
 
         S_ext = s2s(S_ext, self.port_z0, S_DEF_DEFAULT, 'traveling')
         return S_ext  # shape (nb_frequency, nb_ports, nb_ports)
@@ -884,13 +938,8 @@ class Circuit:
         ntw : :class:`~skrf.network.Network`
             Network associated to external ports
         """
-        ntw = Network()
-        ntw.frequency = self.frequency
-        ntw.z0 = self.port_z0
-        ntw.s = self.s_external
-        ntw.name = self.name
-        return ntw
-
+        return Network(frequency = self.frequency, z0 = self.port_z0,
+                      s = self.s_external, name = self.name)
 
     def s_active(self, a: NumberLike) -> np.ndarray:
         r"""
@@ -1203,7 +1252,11 @@ class Circuit:
         b = self._b(a)
         z0s = self.z0
         directions = self._currents_directions
-        Is = (b[:,directions[:,0]] - b[:,directions[:,1]])/np.sqrt(z0s)
+        i_l, i_r = directions[:, 0], directions[:, 1]
+
+        z0_sqrt = np.sqrt(z0s)
+        Is = (b[:,i_l] / z0_sqrt[:,i_r] - b[:,i_r] / z0_sqrt[:,i_l]) \
+            * (2*z0_sqrt[:,i_l] * z0_sqrt[:,i_r]) / (z0s[:,i_l] + z0s[:,i_r])
         return Is
 
 
@@ -1233,7 +1286,11 @@ class Circuit:
         b = self._b(a)
         z0s = self.z0
         directions = self._currents_directions
-        Vs = (b[:,directions[:,0]] + b[:,directions[:,1]])*np.sqrt(z0s)
+        i_l, i_r = directions[:, 0], directions[:, 1]
+
+        z0_sqrt = np.sqrt(z0s)
+        Vs = (b[:,i_l] * z0_sqrt[:,i_r] + b[:,i_r] * z0_sqrt[:,i_l]) \
+            * (2*z0_sqrt[:,i_l] * z0_sqrt[:,i_r]) / (z0s[:,i_l] + z0s[:,i_r])
         return Vs
 
     def currents_external(self, power: NumberLike, phase: NumberLike) -> np.ndarray:
@@ -1400,9 +1457,10 @@ class Circuit:
         fig.tight_layout()
 
 ## Functions operating on Circuit
-def reduce_circuit(connections: list[list[tuple]],
+def reduce_circuit(connections: list[list[tuple[Network, int]]],
                    check_duplication: bool = True,
-                   split_ground: bool = False) -> list[list[tuple]]:
+                   split_ground: bool = False,
+                   max_nports: int = 20) -> list[list[tuple[Network, int]]]:
     """
     Return a reduced equivalent circuit connections with fewer components.
 
@@ -1417,6 +1475,11 @@ def reduce_circuit(connections: list[list[tuple]],
             If True, check if the connections have duplicate names. Default is True.
     split_ground : bool, optional.
             If True, split the global ground connection to independant ground connections. Default is False.
+    max_nports : int
+            The maximum number of ports of a Network that can be reduced in circuit. If a Network in the
+            circuit has a number of ports (nports), using the Network.connect() method to reduce the circuit's
+            dimensions becomes less efficient compared to directly calculating it with Circuit.s_external.
+            This value depends on the performance of the computer and the scale of the circuit. Default is 20.
 
 
     Returns
@@ -1438,8 +1501,9 @@ def reduce_circuit(connections: list[list[tuple]],
     True
     """
 
-    def invalide_to_reduce(cnx):
-        return any(Circuit._is_port(ntwk) for ntwk, _ in cnx) or len(cnx) != 2
+    def invalide_to_reduce(cnx: list[tuple[Network, int]]) -> bool:
+        return any((Circuit._is_port(ntwk) or ntwk.nports > max_nports)
+                   for ntwk, _ in cnx) or len(cnx) != 2
 
     if split_ground:
         tmp_cnxs = []
@@ -1456,22 +1520,38 @@ def reduce_circuit(connections: list[list[tuple]],
                 if Circuit._is_ground(ntwk):
                     continue
                 tmp_gnd = Circuit.Ground(frequency=ground_ntwk.frequency,
-                                         name=f'G_{ntwk.name}_{port}',
-                                         z0=ground_ntwk.z0)
+                                         name=f'G_{ntwk.name}_{port}')
+                tmp_gnd.z0 = ground_ntwk.z0
                 tmp_cnxs.append([(ntwk, port), (tmp_gnd, 0)])
 
         connections = tmp_cnxs
 
-    # get the total number of network ports in the specified connection
+    # Cache the connections that have been processed to avoid loop
+    processed_network_names: set[str] = set()
+
+    # Calculate the total number of Network ports in the specified connection
     def calculate_ports(cnx: list[tuple[Network, int]]) -> int:
         if invalide_to_reduce(cnx):
             return -1
 
-        unique_networks = len(set(ntwk.name for ntwk, _ in cnx))
+        name_list = sorted(ntwk.name for ntwk, _ in cnx)
+        ntwks_str: str = ''.join(name_list)
+
+        unique_networks = len(set(name_list))
         total_ports = sum(ntwk.nports for ntwk, _ in cnx) - 2
 
         # Return the number of ports if the connections performed
-        return total_ports if unique_networks == 2 else total_ports // 2 - 1
+        ports = total_ports if unique_networks == 2 else total_ports // 2 - 1
+
+        # If tuples of Networks in 'connections' have the same Networks, they form a loop.
+        # Prioritize processing loops in the circuit by reducing the number of ports by 1.
+        # This reduces the computational load during the circuit reduction process.
+        if ntwks_str in processed_network_names:
+            ports -= 1
+        else:
+            processed_network_names.add(ntwks_str)
+
+        return ports
 
     # List of tuples containing connection indices and their calculated ports
     cnx_ports_list = [(idx, calculate_ports(cnx)) for idx, cnx in enumerate(connections)]
