@@ -83,6 +83,36 @@ class CircuitTestConstructor(unittest.TestCase):
         reduced_circuit = rf.Circuit(connections, auto_reduce=True)
         assert_array_almost_equal(full_circuit.s_external, reduced_circuit.s_external)
 
+    def test_cache_attributes(self):
+        """
+        Test the cached attributes of the Circuit
+        """
+        connections = [[(self.port1, 0), (self.ntwk1, 0)],
+                       [(self.ntwk1, 1), (self.ntwk2, 0)],
+                       [(self.ntwk2, 1), (self.port2, 0)]]
+
+        init_circuit = rf.Circuit(connections)
+
+        cached_attributes = ('s', 'X', 'C')
+
+        # Initial circuit should not have cached attributes
+        for attr in cached_attributes:
+            self.assertFalse(init_circuit.__dict__.get(attr, False))
+
+        # Access the attributes to cache them
+        X = init_circuit.X
+        C = init_circuit.C
+        s = init_circuit.s
+
+        # Check that the cached attributes are set correctly
+        for attr, value in zip(cached_attributes, (s, X, C)):
+            assert_array_almost_equal(init_circuit.__dict__.get(attr, 0.0), value)
+
+        # Modify connections to invalidate the cache
+        init_circuit.connections = connections
+        for attr in cached_attributes:
+            self.assertFalse(init_circuit.__dict__.get(attr, False))
+
 class CircuitClassMethods(unittest.TestCase):
     """
     Test the various class methods of Circuit such as Ground, Port, etc.
@@ -101,8 +131,7 @@ class CircuitClassMethods(unittest.TestCase):
 
         gnd = rf.Circuit.Ground(self.freq, 'gnd')
         gnd_ref = rf.Network(frequency=self.freq,
-                             s=np.tile(np.array([[-1, 0],
-                                                 [0, -1]]),
+                             s=np.tile(np.array([[-1,]]),
                                        (len(self.freq),1,1)))
 
         assert_array_almost_equal(gnd.s, gnd_ref.s)
@@ -118,8 +147,7 @@ class CircuitClassMethods(unittest.TestCase):
 
         opn = rf.Circuit.Open(self.freq, 'open')
         opn_ref = rf.Network(frequency=self.freq,
-                             s=np.tile(np.array([[1, 0],
-                                                 [0, 1]]),
+                             s=np.tile(np.array([[1]]),
                                        (len(self.freq),1,1)))
 
         assert_array_almost_equal(opn.s, opn_ref.s)
@@ -150,10 +178,10 @@ class CircuitClassMethods(unittest.TestCase):
                     self.media.shunt(self.media.load(rf.zl_2_Gamma0(z0, 1/Y))).s
                     )
 
-        # Y=INF is a a 2-ports short, aka a ground
+        # Y=INF is a a 2-ports short
         assert_array_almost_equal(
             rf.Circuit.ShuntAdmittance(self.freq, rf.INF, 'imp').s,
-            rf.Circuit.Ground(self.freq, 'ground').s
+            self.media.short(nports=2).s
             )
 
 class CircuitTestWilkinson(unittest.TestCase):
@@ -1061,6 +1089,29 @@ class CircuitTestVoltagesCurrents(unittest.TestCase):
         # (toward the Circuit's Port)
         np.testing.assert_allclose(self.I_out, -1*I_ports[:,1])
 
+    def test_tline_with_different_impedance(self):
+        ' Test voltages and currents for a simple transmission line with different impedances '
+        line = self.line.copy()
+        line.renormalize(z_new=1./self.Z)
+
+        # Equivalent model with Circuit
+        port1 = rf.Circuit.Port(frequency=self.freq, name='port1', z0=self.Z)
+        port2 = rf.Circuit.Port(frequency=self.freq, name='port2', z0=self.Z)
+        cnx = [
+            [(port1, 0), (line, 0)],
+            [(port2, 0), (line, 1)]
+        ]
+        crt = rf.Circuit(cnx)
+
+        V_ports_uni_z = crt.voltages(self.power, self.phase)
+        I_ports_uni_z = crt.currents(self.power, self.phase)
+
+        V_ports_dif_z = self.crt.voltages(self.power, self.phase)
+        I_ports_dif_z = self.crt.currents(self.power, self.phase)
+
+        np.testing.assert_allclose(I_ports_uni_z, I_ports_dif_z)
+        np.testing.assert_allclose(V_ports_uni_z, V_ports_dif_z)
+
 class CircuitTestVoltagesNonReciprocal(unittest.TestCase):
     def test_isolator(self):
         # Isolator that passes from port 1 to 2
