@@ -62,6 +62,7 @@ class NetworkTestCase(unittest.TestCase):
         l1 = self.cpw.line(0.20, 'm', z0=50)
         l2 = self.cpw.line(0.07, 'm', z0=50)
         l3 = self.cpw.line(0.47, 'm', z0=50)
+        self.l2 = l2
         self.Fix = rf.concat_ports([l1, l1, l1, l1])
         self.DUT = rf.concat_ports([l2, l2, l2, l2])
         self.Meas = rf.concat_ports([l3, l3, l3, l3])
@@ -109,6 +110,12 @@ class NetworkTestCase(unittest.TestCase):
         empty_network = n[n.f < 0]
         self.assertIn('1-Port Network', repr(empty_network))
 
+    def test_network_sequence_frequency_with_f_unit(self):
+        n=rf.Network(frequency=self.freq.f, f_unit=self.freq.unit)
+        np.allclose(n.f, self.freq.f)
+        n=rf.Network(f=self.freq.f, f_unit=self.freq.unit)
+        np.allclose(n.f, self.freq.f)
+
     def test_timedomain(self):
         t = self.ntwk1.s11.s_time
         s = self.ntwk1.s11.s
@@ -150,6 +157,59 @@ class NetworkTestCase(unittest.TestCase):
         gated = ntwk.s11.time_gate()
         self.assertTrue(len(gated)== len(ntwk))
 
+    def test_lpi(self):
+        """Test low pass impulse response against data generated with METAS VNA Tools."""
+
+        path = Path(self.test_dir) / "metas_tdr"
+
+        for fname in ["short_10ps_dc_50g", "short_10ps_dc_40g"]:
+
+            netw = rf.Network(path / f"{fname}.s1p")
+            ref = np.loadtxt(path / f"{fname}_low_pass_impulse.csv", skiprows=1, delimiter=";")
+            t, y = netw.impulse_response(window="boxcar", pad=0, squeeze=True)
+
+            np.testing.assert_allclose(ref[:,0], t * 1e12, rtol=2e-5)
+            np.testing.assert_allclose(ref[:,1], y, rtol=5e-5)
+
+    def test_lps(self):
+        """Test low pass step response against data generated with METAS VNA Tools."""
+        path = Path(self.test_dir) / "metas_tdr"
+
+        for fname in ["short_10ps_dc_50g", "short_10ps_dc_40g"]:
+            netw = rf.Network(path / f"{fname}.s1p")
+            ref = np.loadtxt(path / f"{fname}_low_pass_step.csv", skiprows=1, delimiter=";")
+            t, y = netw.step_response(window="boxcar", pad=0, squeeze=True)
+
+            np.testing.assert_allclose(ref[:, 0], t * 1e12, rtol=2e-5)
+            np.testing.assert_allclose(ref[:, 1], y, rtol=5e-5)
+
+    def test_bpi(self):
+        """Test band pass impulse response against data generated with METAS VNA Tools."""
+        path = Path(self.test_dir) / "metas_tdr"
+        for window in ["boxcar", None]:
+            # Check if window=None equals to window="boxcar"
+            for fname in ["short_10ps_dc_50g", "short_10ps_dc_40g", "short_10ps_10g_50g", "short_10ps_10g_40g"]:
+
+                netw = rf.Network(path / f"{fname}.s1p")
+                ref = np.loadtxt(path / f"{fname}_band_pass_impulse.csv", skiprows=1, delimiter=";")
+                t, y = netw.impulse_response(window=window, pad=0, squeeze=True, bandpass=True)
+
+                np.testing.assert_allclose(ref[:,0], t * 1e12, rtol=2e-5)
+                np.testing.assert_allclose(ref[:,1], np.abs(y), atol=1e-5)
+
+    def test_auto_use_bandpass(self):
+        path = Path(self.test_dir) / "metas_tdr"
+
+        for fname in ["short_10ps_dc_50g", "short_10ps_10g_50g"]:
+
+            netw = rf.Network(path / f"{fname}.s1p")
+            t, _y = netw.impulse_response(window="boxcar", pad=0, squeeze=True)
+            if netw.frequency.start == 0:
+                assert len(t) == 2 * len(netw) - 1
+            else:
+                assert len(t) == len(netw)
+
+
 
     def test_time_transform_v2(self):
         spb = (4, 5)
@@ -185,14 +245,11 @@ class NetworkTestCase(unittest.TestCase):
         s = np.ones(10)
         netw = rf.Network(frequency=freq, s=s)
 
-        n_lst = np.arange(-1,2) + 2 * (f_points) - 2
-        for n in n_lst:
-            t,y = netw.impulse_response('boxcar', n=n)
-
-            y_true = np.zeros_like(y)
-            y_true[t == 0] = 1
-            np.testing.assert_almost_equal(y, y_true)
+        t, y = netw.impulse_response("boxcar", pad=0)
 
+        y_true = np.zeros_like(y)
+        y_true[t == 0] = 1
+        np.testing.assert_almost_equal(y, y_true)
 
     def test_time_transform_nonlinear_f(self):
         netw_nonlinear_f = rf.Network(os.path.join(self.test_dir, 'ntwk_arbitrary_frequency.s2p'))
@@ -343,6 +400,12 @@ class NetworkTestCase(unittest.TestCase):
         self.ntwk1.write_touchstone('ntwk1Saved',dir=self.test_dir)
         ntwk1Saved = rf.Network(os.path.join(self.test_dir, 'ntwk1Saved.s2p'))
         self.assertEqual(self.ntwk1, ntwk1Saved)
+
+        # Test that it still works with Pathlib objects
+        self.ntwk1.write_touchstone(Path('ntwk1Saved'),dir=Path(self.test_dir))
+        ntwk1Saved = rf.Network(Path(os.path.join(self.test_dir, 'ntwk1Saved.s2p')))
+        self.assertEqual(self.ntwk1, ntwk1Saved)
+
         os.remove(os.path.join(self.test_dir, 'ntwk1Saved.s2p'))
 
     def test_write_touchstone(self):
@@ -441,6 +504,13 @@ class NetworkTestCase(unittest.TestCase):
         self.assertEqual(self.ntwk1 >> self.ntwk2, self.ntwk3)
         self.assertEqual(self.Fix2 >> self.DUT2 >> self.Fix2.flipped(), self.Meas2)
 
+    def test_concat_ports(self):
+        for idx in range(4):
+            i,j = 2*idx, 2*(idx+1)
+            self.assertTrue(np.allclose(self.DUT2.s[:, i:j, i:j], self.l2.s)) # check s-parameters
+            self.assertTrue(np.allclose(self.DUT2.z0[:, i:j], self.l2.z0)) # check z0
+        self.assertTrue(np.all(self.DUT2.port_modes == np.array(['S']*8))) # check port mode
+
     def test_connect(self):
         self.assertEqual(rf.connect(self.ntwk1, 1, self.ntwk2, 0) , \
             self.ntwk3)
@@ -460,7 +530,7 @@ class NetworkTestCase(unittest.TestCase):
         line = med.line(1, unit='m')
         line.z0 = [10, 20]
 
-        for nport_portnum in [3,4,5,6,7,8]:
+        for nport_portnum in [1,2,3,4,5,6,7,8]:
 
             # create a Nport network with port impedance i at port i
             nport = rf.Network()
@@ -744,7 +814,7 @@ class NetworkTestCase(unittest.TestCase):
         ntwk.port_names = from_ports_name
         ntwk_renum = ntwk.renumbered(from_ports_num, to_ports_num)
 
-        np.array_equal(to_ports_name, ntwk_renum.port_names)
+        assert to_ports_name == ntwk_renum.port_names
 
     def test_de_embed_by_inv(self):
         self.assertEqual(self.ntwk1.inv ** self.ntwk3, self.ntwk2)
@@ -1066,6 +1136,8 @@ class NetworkTestCase(unittest.TestCase):
         ntwk_orig.write_touchstone(os.path.join(self.test_dir, pwfile_skrf), write_z0=True, form='RI')
         ntwk_skrf = rf.Network(os.path.join(self.test_dir, pwfile_skrf))
 
+        # check if the s_def could be correctly recovered from scikit-rf's Touchstone file
+        self.assertTrue(ntwk_orig.s_def == ntwk_skrf.s_def)
         self.assertTrue(ntwk_orig == ntwk_skrf)
 
     def test_network_from_z_or_y(self):
@@ -1255,6 +1327,14 @@ class NetworkTestCase(unittest.TestCase):
         self.assertTrue(b.z0[1] == 0.5*(a.z0[0] + a.z0[1]))
         self.assertTrue(b.z0[-1] == a.z0[-1])
 
+    def test_interpolate_freq_cropped(self):
+        a = rf.N(f=np.arange(20), s=np.arange(20)*(1+1j),z0=1, f_unit="ghz")
+        freq = rf.F.from_f(np.linspace(1,2,3,endpoint=True), unit='GHz')
+        for method in ('linear', 'cubic', 'quadratic', 'rational'):
+            b = a.interpolate(freq, freq_cropped=False, kind=method)
+            c = a.interpolate(freq, kind=method)
+            self.assertTrue(np.allclose(b.s, c.s))
+
     def test_interpolate_self(self):
         """Test resample."""
         a = rf.N(f=[1,2], s=[1+2j, 3+4j], z0=1)
@@ -1960,5 +2040,27 @@ class NetworkTestCase(unittest.TestCase):
         with pytest.raises(ValueError):
             ntwk_result_3 = self.ntwk1 // (self.ntwk1, self.ntwk2, self.ntwk3)
 
+    def test_fmt_trace_name(self):
+        # Test trace name of differential thru
+        s = np.zeros((1,4,4), dtype=complex)
+        s[:,2,0] = 1
+        s[:,0,2] = 1
+        s[:,3,1] = 1
+        s[:,1,3] = 1
+        # single-ended
+        se_thru = rf.Network(s=s, f=[1], z0=50)
+        self.assertTrue(np.all(se_thru.port_modes == "S"))
+        self.assertTrue(se_thru._fmt_trace_name(0, 0) == "11")
+        self.assertTrue(se_thru._fmt_trace_name(1, 0) == "21")
+        mm_thru = se_thru.copy()
+        mm_thru.se2gmm(p=2)
+        self.assertTrue(np.all(mm_thru.port_modes == ["D", "D", "C", "C"]))
+        self.assertTrue(mm_thru._fmt_trace_name(0, 0) == "dd11")
+        self.assertTrue(mm_thru._fmt_trace_name(1, 0) == "dd21")
+        self.assertTrue(mm_thru._fmt_trace_name(2, 2) == "cc33")
+        self.assertTrue(mm_thru._fmt_trace_name(3, 2) == "cc43")
+        self.assertTrue(mm_thru._fmt_trace_name(2, 0) == "cd31")
+        self.assertTrue(mm_thru._fmt_trace_name(1, 3) == "dc24")
+
 suite = unittest.TestLoader().loadTestsFromTestCase(NetworkTestCase)
 unittest.TextTestRunner(verbosity=2).run(suite)
